# State::Machine::Transition Unknown Failure Class
package State::Machine::Failure::Transition::Unknown;

use Bubblegum::Class;

extends 'State::Machine::Failure';

our $VERSION = '0.04'; # VERSION

1;
