# State::Machine::Transition Execution Failure Class
package State::Machine::Failure::Transition::Execution;

use Bubblegum::Class;

extends 'State::Machine::Failure::Transition';

our $VERSION = '0.04'; # VERSION

1;
